package com.seeyon.cip.connector.auth.template;

import com.seeyon.boot.exception.BusinessException;
import com.seeyon.boot.transport.BaseDto;
import com.seeyon.boot.util.JsonUtils;
import com.seeyon.boot.util.encrypt.MD5Utils;
import com.seeyon.boot.util.http.HttpClientUtil;
import com.seeyon.boot.util.http.common.HttpResult;
import com.seeyon.boot.util.validate.Validators;
import com.seeyon.cip.connector.api.authentication.LinkerSecurityService;
import com.seeyon.cip.connector.dto.authentication.AuthEventDto;
import com.seeyon.cip.connector.dto.authentication.AuthenticationDto;
import com.seeyon.cip.connector.dto.authentication.AuthenticationUserDto;
import com.seeyon.cip.connector.enums.AuthInfoModeEnum;
import lombok.Getter;
import lombok.Setter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.validation.constraints.NotBlank;
import java.nio.charset.StandardCharsets;
import java.util.Base64;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

/**
 * @author wangl
 * @version 1.0
 * @description: 连接器安全SPI开发模板
 * @date 2023/12/28 17:04
 */
@Service
@Slf4j
public class TemplateSecurityServiceImpl implements LinkerSecurityService {

    /**
     * 认证类型名称,需要全英文
     * 需要保证名称唯一,名称+SecurityService 为当前类名称，以当前template为例，实现类名称为 TemplateSecurityServiceImpl
     * @return 类型名称
     */
    @Override
    public String getName() {
        return "template";
    }

    /**
     * 认证类型中文名称,页面选择展示
     * 要求唯一
     * @return 认证类型中文名称
     */
    @Override
    public String getTypeCaption() {
        return "模板认证";
    }

    /**
     * 页面 认证机制枚举
     * 需要当前用户级别信息选用户级别
     * TENANT: 租户级别
     * USER: 用户级别 会在doAuthentication增加当前登录人的用户信息
     * @return 认证模式
     */
    @Override
    public AuthInfoModeEnum getMode() {
        return AuthInfoModeEnum.USER;
    }

    /**
     * 认证请求(核心)
     * @param authenticationDto 安全数据
     * authId: 该安全认证在数据库中id
     * detailId: 动作id,外部正在执行的动作id
     * json: 页面配置动态数据,自行转换对应dto或者map
     * params: 请求参数
     * user: 当前登录人用户信息(包含用户映射配置信息)
         * innerUserId: 内部用户ID
         * innerUserName: 内部用户名称
         * innerUserCODE: 内部用户编号
         * outerUser: 外部用户账户
         * outerUserId: 外部用户ID
         * innerUserLoginName: 内部用户登录名
         * innerUserEmail: 内部用户邮箱
         * innerUserPhone: 内部用户手机号
         * innerUserOfficeNumber: 内部用户办公电话
         * innerUserBankAccount: 内部用户银行账户
         * innerUserOrgId: 内部用户组织编号
     * @return 请求map  <位置-->Map<key,value>> 位置:header query body
     * 如:  {"body":{"token":"123"}}
     */
    @Override
    public Map<String, Object> doAuthentication(AuthenticationDto authenticationDto) throws Exception{
        // 转换页面动态配置信息为dto
        AuthDto authDto = JsonUtils.fromJson(authenticationDto.getJson(), AuthDto.class);
        AuthenticationUserDto user = authenticationDto.getUser();
        JsonDto jsonDto = new JsonDto();
        String username = null;
        if(user.getUserMap().get(authDto.getUsernameParam()) == null){
            throw new BusinessException("安全登录:用户名为空");
        }else {
            username = user.getUserMap().get(authDto.getUsernameParam()).toString();
            jsonDto.setXxx3(username);
        }
        if(user.getUserMap().get(authDto.getPasswordParam()) == null){
            throw new BusinessException("安全登录:密码为空");
        }else {
            jsonDto.setXxx4( MD5Utils.md5(user.getUserMap().get(authDto.getPasswordParam()).toString()).toLowerCase(Locale.ROOT));
        }
        String json = JsonUtils.toJson(jsonDto);
        String base64 = Base64.getEncoder().encodeToString(json.getBytes(StandardCharsets.UTF_8));
        log.info("安全登录,请求参数加密前:{}",json);
        log.info("安全登录,请求参数加密后:{}",base64);
        HttpResult httpResult = HttpClientUtil.sendPost(authDto.getTokenUrl(), base64);
        if(httpResult.getStatusCode()!= 200){
            throw new BusinessException("安全登录,请求返回结果码:{}", String.valueOf(httpResult.getStatusCode()));
        }
        log.info("安全登录,请求返回结果:{}",httpResult.getResult());
        AuthTokenResponse authTokenResponse = JsonUtils.fromJson(httpResult.getResult(), AuthTokenResponse.class);
        if(!"success".equalsIgnoreCase(authTokenResponse.getErrInfo())){
            throw new BusinessException("安全登录,请求返回结果:{}",httpResult.getResult());
        }
        HashMap<String, Object> map = new HashMap<>();
        map.put("user",username);
        map.put("token", authTokenResponse.getToken());
        if("param".equals(authDto.getParameterLocation())){
            map.put("projectName",authDto.getProjectName());
        }else{
            map.put("project_name",authDto.getProjectName());
        }
        HashMap<String, Object> result = new HashMap<>();
        result.put(authDto.getParameterLocation(),map);
        return result;
    }

    /**
     * 回调解密
     * @param json
     * @return 解密后的回调类型
     */
    @Override
    public AuthEventDto doDecode(String json, AuthEventDto authEventDto) {
        return authEventDto;
    }

    /**
     * 获取页面json语句
     * @return 页面动态参数json
     * 以一个参数为例：
     * {
     *     "caption": "模板",
     *     "type": "template",
     *     "extensionProperties":
     *     [
     *         {
     *             "colProps":
     *             {
     *                 "span": 12
     *             },
     *             "componentType": "Input",
     *             "componentProps":
     *             {
     *                 "placeholder": "请输入"
     *             },
     *             "rules":
     *             [
     *                 {
     *                     "required": true,
     *                     "message": "请输入参数位置"
     *                 }
     *             ],
     *             "caption": "参数位置",
     *             "defaultValue": "",
     *             "name": "parameterLocation"
     *         }
     *     ]
     * }
     */
    @Override
    public String getPageJson() {
        return "{\"caption\":\"模板\",\"type\":\"template\",\"extensionProperties\":[{\"colProps\":{\"span\":12},\"componentType\":\"Input\",\"componentProps\":{\"placeholder\":\"请输入\"},\"rules\":[{\"required\":true,\"message\":\"请输入参数位置\"}],\"caption\":\"参数位置\",\"defaultValue\":\"\",\"name\":\"parameterLocation\"},{\"colProps\":{\"span\":12},\"componentType\":\"Input\",\"componentProps\":{\"placeholder\":\"请输入\"},\"rules\":[{\"required\":true,\"message\":\"请输入token请求地址\"}],\"caption\":\"token请求地址\",\"defaultValue\":\"\",\"name\":\"tokenUrl\"},{\"colProps\":{\"span\":12},\"componentType\":\"Input\",\"componentProps\":{\"placeholder\":\"请输入\"},\"rules\":[{\"required\":true,\"message\":\"请输入项目名称\"}],\"caption\":\"项目名称\",\"defaultValue\":\"\",\"name\":\"projectName\"},{\"colProps\":{\"span\":12},\"componentType\":\"Input\",\"componentProps\":{\"placeholder\":\"请输入\"},\"rules\":[{\"required\":true,\"message\":\"请输入用户参数\"}],\"caption\":\"用户参数\",\"defaultValue\":\"\",\"name\":\"usernameParam\"},{\"colProps\":{\"span\":12},\"componentType\":\"Input\",\"componentProps\":{\"placeholder\":\"请输入\"},\"rules\":[{\"required\":true,\"message\":\"请输入密码参数\"}],\"caption\":\"密码参数\",\"defaultValue\":\"\",\"name\":\"passwordParam\"}]}";
    }

    /**
     * 排序
     * @return 排序号
     */
    @Override
    public Integer getSortNo() {
        return 99;
    }

    /**
     * jsr303校验
     * @param json 页面填写的数据
     */
    @Override
    public void jsonCheck(String json) {
        AuthDto authDto = JsonUtils.fromJson(json, AuthDto.class);
        Validators.assertjsr303(authDto);
    }

    /**
     * 校验执行动作最终结果
     * @param authenticationDto 安全数据
     * authId: 该安全认证在数据库中id
     * detailId: 动作id,外部正在执行的动作id
     * json: 页面配置动态数据,自行转换对应dto或者map
     * params: 请求参数
     * user: 当前登录人用户信息(包含用户映射配置信息)
     * innerUserId: 内部用户ID
     * innerUserName: 内部用户名称
     * innerUserCODE: 内部用户编号
     * outerUser: 外部用户账户
     * outerUserId: 外部用户ID
     * innerUserLoginName: 内部用户登录名
     * innerUserEmail: 内部用户邮箱
     * innerUserPhone: 内部用户手机号
     * innerUserOfficeNumber: 内部用户办公电话
     * innerUserBankAccount: 内部用户银行账户
     * innerUserOrgId: 内部用户组织编号
     * @return true: 需要重试  null:不需要验证 false:不需要重试
     */
    @Override
    public Boolean doVerification(AuthenticationDto authenticationDto) {
        return null;
    }

    @Getter
    @Setter
    public static class JsonDto {
        private String xxx1 = "GetToken";
        private String xxx2 = "";
        private String xxx3 ;
        private String xxx4 ;
    }

    @Getter
    @Setter
    public static class AuthDto extends BaseDto {
        //参数位置
        @NotBlank
        private String parameterLocation;
        //token请求地址
        @NotBlank
        private String tokenUrl;
        //项目名称
        @NotBlank
        private String projectName;
        //用户参数
        @NotBlank
        private String usernameParam;
        //密码参数
        @NotBlank
        private String passwordParam;
    }

    @Getter
    @Setter
    public static class AuthTokenResponse extends BaseDto {
        private String errInfo;
        private String project;
        private String rescode;
        private String token;
    }
}
